@@ -46,6 +46,14 @@ get_object_signature (GObject *obj)
     DBUS_STRUCT_END_CHAR_AS_STRING;
 }
 
+static void
+append_strv (DBusMessageIter *iter, char **strv)
+{
+  _g_dbus_message_iter_append_args (iter,
+				    DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &strv, g_strv_length (strv),
+				    0);
+}
+
 static void
 append_object (DBusMessageIter *iter, GObject *obj)
 {
@@ -100,6 +108,9 @@ _g_dbus_attribute_value_destroy (GFileAttributeType          type,
   case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
     g_free (value->ptr);
     break;
+  case G_FILE_ATTRIBUTE_TYPE_STRINGV:
+    g_strfreev (value->ptr);
+    break;
   case G_FILE_ATTRIBUTE_TYPE_OBJECT:
     if (value->ptr)
       g_object_unref (value->ptr);
@@ -136,6 +147,9 @@ _g_dbus_type_from_file_attribute_type (GFileAttributeType type)
     case G_FILE_ATTRIBUTE_TYPE_BYTE_STRING:
       dbus_type = DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_BYTE_AS_STRING;
       break;
+    case G_FILE_ATTRIBUTE_TYPE_STRINGV:
+      dbus_type = DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING;
+      break;
     case G_FILE_ATTRIBUTE_TYPE_BOOLEAN:
       dbus_type = DBUS_TYPE_BOOLEAN_AS_STRING;
       break;
@@ -154,6 +168,9 @@ _g_dbus_type_from_file_attribute_type (GFileAttributeType type)
     case G_FILE_ATTRIBUTE_TYPE_OBJECT:
       dbus_type = DBUS_TYPE_STRUCT_AS_STRING;
       break;
+    case G_FILE_ATTRIBUTE_TYPE_INVALID:
+      dbus_type = DBUS_TYPE_BYTE_AS_STRING;
+      break;
     default:
       dbus_type = NULL;
       g_warning ("Invalid attribute type %u, ignoring\n", type);
@@ -166,12 +183,14 @@ _g_dbus_type_from_file_attribute_type (GFileAttributeType type)
 void
 _g_dbus_append_file_attribute (DBusMessageIter *iter,
 			       const char *attribute,
+			       GFileAttributeStatus status,
 			       GFileAttributeType type,
 			       gpointer value_p)
 {
   DBusMessageIter variant_iter, inner_struct_iter;
   const char *dbus_type;
   GObject *obj = NULL;
+  guint32 dbus_status;
 
   dbus_type = _g_dbus_type_from_file_attribute_type (type);
 
@@ -186,6 +205,12 @@ _g_dbus_append_file_attribute (DBusMessageIter *iter,
 				       &attribute))
     _g_dbus_oom ();
 
+  dbus_status = status;
+  if (!dbus_message_iter_append_basic (&inner_struct_iter,
+				       DBUS_TYPE_UINT32,
+				       &dbus_status))
+    _g_dbus_oom ();
+
   if (dbus_type[0] == DBUS_TYPE_STRUCT)
     dbus_type = get_object_signature ((GObject *)value_p);
   
@@ -201,8 +226,17 @@ _g_dbus_append_file_attribute (DBusMessageIter *iter,
 					   DBUS_TYPE_STRING, &value_p))
 	_g_dbus_oom ();
     }
-  else if (dbus_type[0] == DBUS_TYPE_ARRAY)
+  else if (dbus_type[0] == DBUS_TYPE_ARRAY && dbus_type[1] == DBUS_TYPE_BYTE)
     _g_dbus_message_iter_append_cstring (&variant_iter, (char *)value_p);
+  else if (dbus_type[0] == DBUS_TYPE_ARRAY && dbus_type[1] == DBUS_TYPE_STRING)
+    append_strv (&variant_iter, (char **)value_p);
+  else if (dbus_type[0] == DBUS_TYPE_BYTE)
+    {
+      char byte = 0;
+      if (!dbus_message_iter_append_basic (&variant_iter,
+					   DBUS_TYPE_BYTE, &byte))
+	_g_dbus_oom ();
+    }
   else if (dbus_type[0] == DBUS_STRUCT_BEGIN_CHAR)
     append_object (&variant_iter, (GObject *)value_p);
   else if (dbus_type[0] == DBUS_TYPE_BOOLEAN)
@@ -252,6 +286,7 @@ _g_dbus_append_file_info (DBusMessageIter *iter,
 					 DBUS_TYPE_ARRAY,
 					 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
 					 DBUS_TYPE_STRING_AS_STRING	
+					 DBUS_TYPE_UINT32_AS_STRING
 					 DBUS_TYPE_VARIANT_AS_STRING	
 					 DBUS_STRUCT_END_CHAR_AS_STRING,
 					 &array_iter))
@@ -260,10 +295,11 @@ _g_dbus_append_file_info (DBusMessageIter *iter,
   for (i = 0; attributes[i] != NULL; i++)
     {
       GFileAttributeType type;
+      GFileAttributeStatus status;
       gpointer value_p;
 
-      if (g_file_info_get_attribute_data (info, attributes[i], &type, &value_p, NULL))
-	_g_dbus_append_file_attribute (&array_iter, attributes [i], type, value_p);
+      if (g_file_info_get_attribute_data (info, attributes[i], &type, &value_p, &status))
+	_g_dbus_append_file_attribute (&array_iter, attributes [i], status, type, value_p);
     }
   
   g_strfreev (attributes);
@@ -278,6 +314,7 @@ _g_dbus_append_file_info (DBusMessageIter *iter,
 gboolean
 _g_dbus_get_file_attribute (DBusMessageIter *iter,
 			    gchar **attribute,
+			    GFileAttributeStatus *status,
 			    GFileAttributeType *type,
 			    GDbusAttributeValue *value)
 {
@@ -286,12 +323,14 @@ _g_dbus_get_file_attribute (DBusMessageIter *iter,
   int n_elements;
   DBusMessageIter inner_struct_iter, variant_iter, cstring_iter, obj_iter;
   const gchar *attribute_temp;
-  dbus_uint32_t obj_type;
+  int element_type;
+  dbus_uint32_t obj_type, dbus_status;
   dbus_bool_t dbus_bool;
+  guint8 byte;
   GObject *obj;
 
   dbus_message_iter_recurse (iter, &inner_struct_iter);
-      
+
   if (dbus_message_iter_get_arg_type (&inner_struct_iter) != DBUS_TYPE_STRING)
     goto error;
 	
@@ -299,7 +338,13 @@ _g_dbus_get_file_attribute (DBusMessageIter *iter,
   *attribute = g_strdup (attribute_temp);
 
   dbus_message_iter_next (&inner_struct_iter);
-	
+
+  dbus_message_iter_get_basic (&inner_struct_iter, &dbus_status);
+  if (status)
+    *status = dbus_status;
+
+  dbus_message_iter_next (&inner_struct_iter);
+
   if (dbus_message_iter_get_arg_type (&inner_struct_iter) != DBUS_TYPE_VARIANT)
     goto error;
 
@@ -313,15 +358,34 @@ _g_dbus_get_file_attribute (DBusMessageIter *iter,
       value->ptr = g_strdup (str);
       break;
     case DBUS_TYPE_ARRAY:
-      if (dbus_message_iter_get_element_type (&variant_iter) != DBUS_TYPE_BYTE)
+      element_type = dbus_message_iter_get_element_type (&variant_iter);
+      if (element_type == DBUS_TYPE_BYTE)
+	{
+	  *type = G_FILE_ATTRIBUTE_TYPE_BYTE_STRING;
+
+	  dbus_message_iter_recurse (&variant_iter, &cstring_iter);
+	  dbus_message_iter_get_fixed_array (&cstring_iter,
+					     &str, &n_elements);
+	  value->ptr = g_strndup (str, n_elements);
+	}
+      else if (element_type == DBUS_TYPE_STRING)
+	{
+	  char **strv;
+	  int n_elements;
+	  if (!_g_dbus_message_iter_get_args (iter, NULL,
+					      DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &strv, &n_elements,
+					      0))
+	    goto error;
+	  *type = G_FILE_ATTRIBUTE_TYPE_STRINGV;
+	  value->ptr = strv;
+	}
+      else
 	goto error;
 
-      *type = G_FILE_ATTRIBUTE_TYPE_BYTE_STRING;
-
-      dbus_message_iter_recurse (&variant_iter, &cstring_iter);
-      dbus_message_iter_get_fixed_array (&cstring_iter,
-					 &str, &n_elements);
-      value->ptr = g_strndup (str, n_elements);
+      break;
+    case DBUS_TYPE_BYTE:
+      dbus_message_iter_get_basic (&variant_iter, &byte);
+      *type = G_FILE_ATTRIBUTE_TYPE_INVALID;
       break;
     case DBUS_TYPE_BOOLEAN:
       dbus_message_iter_get_basic (&variant_iter, &dbus_bool);
@@ -418,6 +482,7 @@ _g_dbus_get_file_info (DBusMessageIter *iter,
   DBusMessageIter struct_iter, array_iter;
   gchar *attribute;
   GFileAttributeType type;
+  GFileAttributeStatus status;
   GDbusAttributeValue value;
 
   info = g_file_info_new ();
@@ -434,10 +499,12 @@ _g_dbus_get_file_info (DBusMessageIter *iter,
 
   while (dbus_message_iter_get_arg_type (&array_iter) == DBUS_TYPE_STRUCT)
     {
-      if (!_g_dbus_get_file_attribute (&array_iter, &attribute, &type, &value))
+      if (!_g_dbus_get_file_attribute (&array_iter, &attribute, &status, &type, &value))
         goto error;
 
       g_file_info_set_attribute (info, attribute, type, _g_dbus_attribute_as_pointer (type, &value));
+      if (status)
+	g_file_info_set_attribute_status (info, attribute, status);
 
       g_free (attribute);
       _g_dbus_attribute_value_destroy (type, &value);
@@ -516,7 +583,7 @@ _g_dbus_append_attribute_info_list (DBusMessageIter         *iter,
 					 DBUS_TYPE_ARRAY,
 					 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
 					 DBUS_TYPE_STRING_AS_STRING	
-					 DBUS_TYPE_UINT32_AS_STRING	
+					 DBUS_TYPE_UINT32_AS_STRING
 					 DBUS_TYPE_UINT32_AS_STRING	
 					 DBUS_STRUCT_END_CHAR_AS_STRING,
 					 &array_iter))
